@@ -1,3 +1,6 @@
+"""
+The main Stretchy Tree Matching algorithm, implemented as a class.
+"""
 import ast
 import re
 from pedal.cait.ast_map import AstMap
@@ -34,21 +37,18 @@ def _name_regex(name_id):
 
 class StretchyTreeMatcher:
     """
+    The StretchyTreeMatcher is used to compare a pattern against some
+    student code. It produces a set of potential mappings between them.
 
+    Args:
+        ast_or_code (str or AstNode): The students' code or a valid AstNode from
+            `ast.parse`. If the code has invalid syntax, a SyntaxError
+            will be raised.
+        filename (str): The filename to parse with - only used for error
+            reporting.
+        report (Report): A report to obtain data from.
     """
     def __init__(self, ast_or_code, report, filename="__main__"):
-        """
-        The StretchyTreeMatcher is used to compare a pattern against some
-        student code. It produces a set of potential mappings between them.
-
-        Args:
-            ast_or_code (str or AstNode): The students' code or a valid AstNode from
-                `ast.parse`. If the code has invalid syntax, a SyntaxError
-                will be raised.
-            filename (str): The filename to parse with - only used for error
-                reporting.
-            report (Report): A report to obtain data from.
-        """
         self.report = report
         if isinstance(ast_or_code, str):
             ast_node = ast.parse(ast_or_code, filename)
@@ -62,10 +62,10 @@ class StretchyTreeMatcher:
         else:
             self.root_node = CaitNode(ast_node, _NONE_FIELD, report=self.report)
 
-    def find_matches(self, ast_or_code, filename="__main__", check_meta=True, pre_match=None):
+    def find_matches(self, ast_or_code, filename="__main__", check_meta=True, use_previous=None):
         """
         Args:
-            pre_match:
+            use_previous:
             ast_or_code (str or AstNode): The students' code or a valid AstNode from
                 `ast.parse`. If the code has invalid syntax, a SyntaxError
                 will be raised.
@@ -101,17 +101,17 @@ class StretchyTreeMatcher:
                 other_root_old_field = other_root.field
                 other_root.field = _NONE_FIELD
         matches = self.any_node_match(explore_root, other_root,
-                                      check_meta=check_meta, pre_match=pre_match)
+                                      check_meta=check_meta, use_previous=use_previous)
         explore_root.field = explore_root_old_field
         other_root.field = other_root_old_field
         return matches
 
-    def any_node_match(self, ins_node, std_node, check_meta=True, cut=False, pre_match=None):
+    def any_node_match(self, ins_node, std_node, check_meta=True, cut=False, use_previous=None):
         """
         Finds whether ins_node can be matched to some node in the tree std_node
 
         Args:
-            pre_match:
+            use_previous:
             ins_node:
             std_node:
             check_meta:
@@ -125,21 +125,21 @@ class StretchyTreeMatcher:
         # @TODO: create a more public function that converts ins_node and std_node into CaitNodes
         # TODO: Create exhaustive any_node_match
         # matching: an object representing the mapping and the symbol table
-        matching = self.deep_find_match(ins_node, std_node, check_meta, pre_match=pre_match)
+        matching = self.deep_find_match(ins_node, std_node, check_meta, use_previous=use_previous)
         # if a direct matching is found
         if matching:
             for match in matching:
-                match.match_root = std_node
+                match.match_root = match.mappings[ins_node]
         else:
             matching = []
         #    return matching  # return it
         # if not matching or exhaust:  # otherwise
         # try to matching ins_node to each child of std_node, recursively
         for std_child in std_node.children:
-            matching_c = self.any_node_match(ins_node, std_child, check_meta=check_meta, cut=cut, pre_match=pre_match)
+            matching_c = self.any_node_match(ins_node, std_child, check_meta=check_meta, cut=cut, use_previous=use_previous)
             if matching_c:
                 for match in matching_c:
-                    match.match_root = std_child
+                    match.match_root = match.mappings[ins_node]
                 # return matching
                 matching = matching + matching_c
         if len(matching) > 0:
@@ -147,7 +147,7 @@ class StretchyTreeMatcher:
         return []
 
     def deep_find_match(self, ins_node, std_node, check_meta=True,
-                        pre_match=None):
+                        use_previous=None):
         """
         Finds whether ins_node and matches std_node and whether ins_node's children flexibly match std_node's children
         in order
@@ -155,24 +155,24 @@ class StretchyTreeMatcher:
             ins_node: The instructor ast that should be included in the student AST
             std_node: The student AST that we are searching for the included tree
             check_meta: Flag, if True, check whether the two nodes originated from the same ast field
-            pre_match: If this was part of a previous match...
+            use_previous: If this was part of a previous match...
 
         Returns:
             a mapping of nodes and a symbol table mapping ins_node to std_node, or [] if no mapping was found
         """
         method_name = "deep_find_match_" + type(ins_node.astNode).__name__
         target_func = getattr(self, method_name, self.deep_find_match_generic)
-        return target_func(ins_node, std_node, check_meta, pre_match=pre_match)
+        return target_func(ins_node, std_node, check_meta, use_previous=use_previous)
 
     # noinspection PyPep8Naming
-    def deep_find_match_Name(self, ins_node, std_node, check_meta=True, pre_match=None):
+    def deep_find_match_Name(self, ins_node, std_node, check_meta=True, use_previous=None):
         """
 
         Args:
             ins_node:
             std_node:
             check_meta:
-            pre_match:
+            use_previous:
 
         Returns:
 
@@ -185,11 +185,11 @@ class StretchyTreeMatcher:
         if match[_VAR] and meta_matched:  # if variable
             if type(std_node.astNode).__name__ == "Name":
                 return self.deep_find_match_generic(ins_node, std_node,
-                                                    check_meta=check_meta, ignores=["ctx"], pre_match=pre_match)
+                                                    check_meta=check_meta, ignores=["ctx"], use_previous=use_previous)
         # could else return False, but shallow_match_generic should do this as well
-        elif match[_EXP]:  # and meta_matched:  # if expression
+        elif match[_EXP] and meta_matched:  # and meta_matched:  # if expression
             # terminate recursion, the whole subtree should match since expression nodes match to anything
-            mapping.merge_map_with(pre_match)
+            mapping.merge_map_with(use_previous)
             mapping.add_exp_to_sym_table(ins_node, std_node)
             matched = True
         elif match[_WILD] and meta_matched:  # if wild card, don't care
@@ -201,17 +201,17 @@ class StretchyTreeMatcher:
             return [mapping]
         # else
         return self.deep_find_match_generic(ins_node, std_node,
-                                            check_meta=check_meta, ignores=["ctx"], pre_match=pre_match)
+                                            check_meta=check_meta, ignores=["ctx"], use_previous=use_previous)
 
     # noinspection PyPep8Naming
-    def deep_find_match_BinOp(self, ins_node, std_node, check_meta=True, pre_match=None):
+    def deep_find_match_BinOp(self, ins_node, std_node, check_meta=True, use_previous=None):
         """
 
         Args:
             ins_node:
             std_node:
             check_meta:
-            pre_match:
+            use_previous:
 
         Returns:
 
@@ -220,12 +220,12 @@ class StretchyTreeMatcher:
         op = type(op).__name__
         is_generic = not (op == "Mult" or op == "Add")
         if is_generic:
-            return self.deep_find_match_generic(ins_node, std_node, check_meta, pre_match=pre_match)
+            return self.deep_find_match_generic(ins_node, std_node, check_meta, use_previous=use_previous)
         else:  # this means that the node is clearly commutative
-            return self.deep_find_match_binflex(ins_node, std_node, False, pre_match=pre_match)
+            return self.deep_find_match_binflex(ins_node, std_node, False, use_previous=use_previous)
 
     # noinspection PyMethodMayBeStatic
-    def binflex_helper(self, case_left, case_right, new_mappings, base_mappings, pre_match=None):
+    def binflex_helper(self, case_left, case_right, new_mappings, base_mappings, use_previous=None):
         """
         adds to new_mappings (return/modify by argument) the mappings for both the left and right subtrees as denoted by
         case_left and case_right
@@ -234,26 +234,26 @@ class StretchyTreeMatcher:
             case_right: The mappings for the right opperand
             new_mappings: The new set of mappings to generate
             base_mappings: The original mappings of the binop node
-            pre_match: A mapping passed down from an initial match
+            use_previous: A mapping passed down from an initial match
         Returns:
             None
         """
         if case_left and case_right:
             for case_l in case_left:
-                new_map = base_mappings[0].new_merged_map(case_l).new_merged_map(pre_match)
+                new_map = base_mappings[0].new_merged_map(case_l).new_merged_map(use_previous)
                 for case_r in case_right:
                     both = new_map.new_merged_map(case_r)
                     if not both.has_conflicts():
                         new_mappings.append(both)
 
-    def deep_find_match_binflex(self, ins_node, std_node, check_meta=False, pre_match=None):
+    def deep_find_match_binflex(self, ins_node, std_node, check_meta=False, use_previous=None):
         """
 
         Args:
             ins_node:
             std_node:
             check_meta:
-            pre_match:
+            use_previous:
 
         Returns:
 
@@ -275,17 +275,17 @@ class StretchyTreeMatcher:
             # case 1: ins_left->std_left and ins_right->std_right
             case_left = self.deep_find_match(ins_left, std_left, False)
             case_right = self.deep_find_match(ins_right, std_right, False)
-            self.binflex_helper(case_left, case_right, new_mappings, base_mappings, pre_match=pre_match)
+            self.binflex_helper(case_left, case_right, new_mappings, base_mappings, use_previous=use_previous)
             # case 2: ins_left->std_right and ins_right->std_left
             case_left = self.deep_find_match(ins_left, std_right, False)
             case_right = self.deep_find_match(ins_right, std_left, False)
-            self.binflex_helper(case_left, case_right, new_mappings, base_mappings, pre_match=pre_match)
+            self.binflex_helper(case_left, case_right, new_mappings, base_mappings, use_previous=use_previous)
             if len(new_mappings) == 0:
                 return []
             return new_mappings
         return []
 
-    def deep_find_match_Expr(self, ins_node, std_node, check_meta=True, pre_match=None):
+    def deep_find_match_Expr(self, ins_node, std_node, check_meta=True, use_previous=None):
         """
         An Expression node (not to be confused with expressions denoted by the instructor nodes in Name ast nodes)
         checks whether it should be generic, or not
@@ -293,7 +293,7 @@ class StretchyTreeMatcher:
             ins_node: Instructor ast to find in the student ast
             std_node: Student AST to search for the instructor ast in
             check_meta: flag to check whether the fields of the instructor node and the student node should match
-            pre_match: An AstMap from a previous matching run
+            use_previous: An AstMap from a previous matching run
 
         Returns:
             AstMap: a mapping between the instructor and student asts, or False if such a mapping doesn't exist
@@ -301,7 +301,7 @@ class StretchyTreeMatcher:
         # if check_meta and ins_node.field != std_node.field:
         if not self.metas_match(ins_node, std_node, check_meta):
             return []
-        mapping = AstMap() if pre_match is None else pre_match
+        mapping = AstMap() if use_previous is None else use_previous
         value = ins_node.value
         ast_type = type(value.astNode).__name__
         if ast_type == "Name":
@@ -310,7 +310,7 @@ class StretchyTreeMatcher:
             wild_card = re.compile('^___$')  # /regex
             matched = False
             meta_matched = self.metas_match(ins_node, std_node, check_meta)
-            if exp_match.match(name_id):  # and meta_matched:  # if expression
+            if exp_match.match(name_id) and meta_matched:  # and meta_matched:  # if expression
                 # terminate recursion, the whole subtree should match since expression nodes match to anything
                 mapping.add_exp_to_sym_table(value, std_node)
                 matched = True
@@ -322,7 +322,7 @@ class StretchyTreeMatcher:
                 return [mapping]
         return self.deep_find_match_generic(ins_node, std_node, check_meta)
 
-    def deep_find_match_generic(self, ins_node, std_node, check_meta=True, ignores=None, pre_match=None):
+    def deep_find_match_generic(self, ins_node, std_node, check_meta=True, ignores=None, use_previous=None):
         """
         This first uses shallow match to find a base map (match) from which to
         build off. The algorithm then tracks all the possible mappings that
@@ -343,7 +343,7 @@ class StretchyTreeMatcher:
             std_node: Student AST to search for the instructor ast in
             check_meta: flag to check whether the fields of the instructor node and the student node should match
             ignores: List of fields to ignore in the field match
-            pre_match: a map from a previous match
+            use_previous: a map from a previous match
 
         Returns:
             a mapping between the isntructor and student asts, or [] if such a mapping doesn't exist
@@ -353,7 +353,7 @@ class StretchyTreeMatcher:
         base_mappings = self.shallow_match(ins_node, std_node, check_meta)
         if base_mappings:
             for mapping in base_mappings:
-                mapping.merge_map_with(pre_match)
+                mapping.merge_map_with(use_previous)
             # base case this runs 0 times because no children
             # find each child of ins_node that matches IN ORDER
             base_sibs = [-1]
@@ -577,13 +577,14 @@ class StretchyTreeMatcher:
     def shallow_match_Pass(self, ins_node, std_node, check_meta=True):
         """
         An empty body should match to anything
+        TODO: This should match to an entire body rather than just a single statement in a body
         Args:
             ins_node: Instructor ast to find in the student ast
             std_node: Student AST to search for the instructor ast in
             check_meta: flag to check whether the fields of the instructor node and the student node should match
 
         Returns:
-            list of AstMap: a mapping between the isntructor and student asts, or False if such a mapping doesn't exist
+            list of AstMap: a mapping between the instructor and student asts, or False if such a mapping doesn't exist
         """
         # if check_meta and ins_node.field != std_node.field:
         if not self.metas_match(ins_node, std_node, check_meta):
